package com.shigongli.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.shigongli.common.BaseController;
import com.shigongli.common.ResponseBean;
import com.shigongli.common.StringUtils;
import com.shigongli.constants.StatusConstant;
import com.shigongli.entity.TaHouseOrder;
import com.shigongli.service.ITaHouseOrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.shigongli.service.ITaHouseSettingService;
import com.shigongli.entity.TaHouseSetting;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 房源入住设置 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-12-11
 */

@Api(tags = "房源入住设置")
@RestController
@RequestMapping("/")
public class TaHouseSettingController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaHouseSettingController.class);

    @Autowired
    public ITaHouseSettingService iTaHouseSettingService;

    @Autowired
    ITaHouseOrderService iTaHouseOrderService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/taHouseSetting",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taHouseSettingList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{

		    IPage<TaHouseSetting> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaHouseSetting> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_date");

            IPage<TaHouseSetting> result = iTaHouseSettingService.page(pg, queryWrapper);
            return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taHouseSetting 实体对象
     * @return
     */
    @RequestMapping(value="/ma/taHouseSetting",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taHouseSettingAdd(@ApiParam("保存内容") @RequestBody TaHouseSetting taHouseSetting) throws Exception{
        if (null == taHouseSetting) {
            return ResponseBean.error("填写房源设置出错", ResponseBean.ERROR_MISSING_PARAMS);
        }

        if (null == taHouseSetting.getPersonNum() || taHouseSetting.getPersonNum() < 1) {
            return ResponseBean.error("房源入住人数至少1人", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (StringUtils.isEmpty(taHouseSetting.getHouseId())) {
            return ResponseBean.error("填写选择房源", ResponseBean.ERROR_MISSING_PARAMS);
        }

        // 如果是更新内容
        if (!StringUtils.isEmpty(taHouseSetting.getSettingId())) {
            return taHouseSettingUpdate(taHouseSetting.getSettingId(), taHouseSetting);
        }

        if (iTaHouseSettingService.save(taHouseSetting)){

            // 生成订单
            TaHouseOrder taHouseOrder = new TaHouseOrder();
            taHouseOrder.setSettingId(taHouseSetting.getSettingId());
            taHouseOrder.setStatus(StatusConstant.READY);
            taHouseOrder.setOrderId(iTaHouseOrderService.getNextOrderId());
            iTaHouseOrderService.save(taHouseOrder);
            taHouseSetting.setTaHouseOrder(taHouseOrder);

            return ResponseBean.success(taHouseSetting);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taHouseSetting/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taHouseSettingDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        if(iTaHouseSettingService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taHouseSetting 实体对象
     * @return
     */
    @RequestMapping(value="/taHouseSetting/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taHouseSettingUpdate(@ApiParam("对象ID") @PathVariable String id,
                                             @ApiParam("更新内容") @RequestBody TaHouseSetting taHouseSetting) throws Exception{

        if (iTaHouseSettingService.updateById(taHouseSetting)){
            return ResponseBean.success(iTaHouseSettingService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taHouseSetting/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taHouseSettingGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        return ResponseBean.success(iTaHouseSettingService.getById(id));
    }


    /**
     * 根据房源查询最近的设置记录
     * @param houseId  房源ID
     */
    @RequestMapping(value="/ma/taHouseSetting/last",method= RequestMethod.GET)
    @ApiOperation(value="根据房源查询最近的设置记录", notes = "根据房源查询最近的设置记录", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getTaHouseSettingLast(@ApiParam("房源ID") @RequestParam("houseId") String houseId) throws Exception{
        TaHouseSetting taHouseSetting = iTaHouseSettingService.getLastByHouse(houseId);
        return ResponseBean.success(taHouseSetting);
    }
}
